﻿///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 2011 EaseFilter Technologies
//    All Rights Reserved
//
//    This software is part of a licensed software product and may
//    only be used or copied in accordance with the terms of that license.
//
//    NOTE:  THIS MODULE IS UNSUPPORTED SAMPLE CODE
//
//    This module contains sample code provided for convenience and
//    demonstration purposes only,this software is provided on an 
//    "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
//     either express or implied.  
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Microsoft.Win32.SafeHandles;
using System.Runtime.InteropServices;
using System.Security.Principal;
using System.Threading;
using System.Diagnostics;
using System.Management;
using System.Collections;
using System.Configuration;
using System.Windows.Forms;

using EaseFilter.CommonObjects;

namespace EaseFilter.FolderLocker
{

    public class CacheUserAccessInfo
    {
        public string index = string.Empty;
        public bool accessStatus = false;
        public uint accessFlags = 0;
        public DateTime lastAccessTime = DateTime.MinValue;
        public string iv = string.Empty;
        public string key = string.Empty;
        public AutoResetEvent syncEvent = new AutoResetEvent(true);
        public string lastError = string.Empty;

    }

    public class DRServer
    {

        static Dictionary<string, CacheUserAccessInfo> userAccessCache = new Dictionary<string, CacheUserAccessInfo>();
        static int cacheTimeOutInSeconds = 60;
        static System.Timers.Timer deleteCachedItemTimer = new System.Timers.Timer();

        static DRServer()
        {
            deleteCachedItemTimer.Interval = cacheTimeOutInSeconds * 1000 / 4; //millisecond
            deleteCachedItemTimer.Start();
            deleteCachedItemTimer.Enabled = true;
            deleteCachedItemTimer.Elapsed += new System.Timers.ElapsedEventHandler(deleteCachedItemTimer_Elapsed);
        }

        static private bool GetDRPolicyDataFromByteArray(Byte[] drDataBuffer, uint drBufferLength, ref DRPolicyData drPolicyData, ref string lastError)
        {
            Boolean retVal = false;

            try
            {

                MemoryStream ms = new MemoryStream(drDataBuffer);
                BinaryReader br = new BinaryReader(ms);

                drPolicyData.AESVerificationKey = br.ReadUInt32();

                if (FilterAPI.AES_TAG_KEY != drPolicyData.AESVerificationKey)
                {
                    lastError = "The DRPolicyData buffer was corrupted.";
                    return false;
                }

                drPolicyData.AESFlags = (AESFlags)br.ReadUInt32();
                drPolicyData.IVLength = br.ReadUInt32();
                drPolicyData.IV = br.ReadBytes(16);
                drPolicyData.EncryptionKeyLength = br.ReadUInt32();
                drPolicyData.EncryptionKey = br.ReadBytes(32);
                drPolicyData.CreationTime = br.ReadInt64();
                drPolicyData.ExpireTime = br.ReadInt64();
                drPolicyData.AccessFlags = br.ReadUInt32();
                drPolicyData.FileSize = br.ReadInt64();

                drPolicyData.LengthOfAuthorizedProcessNames = br.ReadUInt32();
                drPolicyData.LengthOfUnauthorizedProcessNames = br.ReadUInt32();
                drPolicyData.LengthOfAuthorizedUserNames = br.ReadUInt32();
                drPolicyData.LengthOfUnauthorizedUserNames = br.ReadUInt32();
                drPolicyData.LengthOfAccountName = br.ReadUInt32();
                drPolicyData.LengthOfComputerIds = br.ReadUInt32();
                drPolicyData.LengthOfUserPassword = br.ReadUInt32();

                if (drPolicyData.LengthOfAuthorizedProcessNames > 0)               
                {
                    byte[] buffer = br.ReadBytes((int)drPolicyData.LengthOfAuthorizedProcessNames);
                    drPolicyData.AuthorizedProcessNames = UnicodeEncoding.Unicode.GetString(buffer);
                }

                if (drPolicyData.LengthOfUnauthorizedProcessNames > 0 )
                {
                    byte[] buffer = br.ReadBytes((int)drPolicyData.LengthOfUnauthorizedProcessNames);
                    drPolicyData.UnauthorizedProcessNames = UnicodeEncoding.Unicode.GetString(buffer);
                }

                if (drPolicyData.LengthOfAuthorizedUserNames > 0 )
                {
                    byte[] buffer = br.ReadBytes((int)drPolicyData.LengthOfAuthorizedUserNames);
                    drPolicyData.AuthorizedUserNames = UnicodeEncoding.Unicode.GetString(buffer);
                }

                if (drPolicyData.LengthOfUnauthorizedUserNames > 0 )
                {
                    byte[] buffer = br.ReadBytes((int)drPolicyData.LengthOfUnauthorizedUserNames);
                    drPolicyData.UnauthorizedUserNames = UnicodeEncoding.Unicode.GetString(buffer);
                }

                if (drPolicyData.LengthOfAccountName > 0)
                {
                    byte[] buffer = br.ReadBytes((int)drPolicyData.LengthOfAccountName);
                    drPolicyData.AccountName = UnicodeEncoding.Unicode.GetString(buffer);
                }

                if (drPolicyData.LengthOfComputerIds > 0)
                {
                    byte[] buffer = br.ReadBytes((int)drPolicyData.LengthOfComputerIds);
                    drPolicyData.ComputerIds = UnicodeEncoding.Unicode.GetString(buffer);
                }

                if (drPolicyData.LengthOfUserPassword > 0 )
                {
                    byte[] buffer = br.ReadBytes((int)drPolicyData.LengthOfUserPassword);
                    drPolicyData.UserPassword = UnicodeEncoding.Unicode.GetString(buffer);
                }

                retVal = true;

            }
            catch (Exception ex)
            {
                retVal = false;
                lastError = "Get DRPolicyData failed with error " + ex.Message;
            }

            return retVal;
        }

        static private bool GetAccessPermissionFromServer(FilterAPI.MessageSendData messageSend,
                                                            DRPolicyData drPolicyData,
                                                            string userName,
                                                            string processName,
                                                            string userPassword,
                                                            ref CacheUserAccessInfo cacheUserAccessInfo)
        {
            Boolean retVal = true;
            string fileName = messageSend.FileName;
            string lastError = string.Empty;
            byte[] computerId = new byte[52];
            GCHandle gcHandle = GCHandle.Alloc(computerId, GCHandleType.Pinned);

            try
            {
               
                UserInfo userInfo = new UserInfo();
                string keyStr = string.Empty;
                string ivStr = string.Empty;

                userInfo.FileName = Path.GetFileName(messageSend.FileName);
                userInfo.AccountName = drPolicyData.AccountName;
                userInfo.ProcessName = processName;
                userInfo.UserName = userName;
                userInfo.UserPassword = userPassword;
                userInfo.CreationTime = drPolicyData.CreationTime;

                ivStr = Utils.ByteArrayToHexStr(drPolicyData.IV);
                userInfo.EncryptionIV = ivStr;

                uint computerIdLength = (uint)computerId.Length;
                IntPtr computerIdPtr = Marshal.UnsafeAddrOfPinnedArrayElement(computerId, 0);
                retVal = FilterAPI.GetUniqueComputerId(computerIdPtr, ref computerIdLength);

                if (!retVal)
                {
                    string message = "Get computerId failed,return error:" + FilterAPI.GetLastErrorMessage();
                    EventManager.WriteMessage(366, "GetAccessPermissionFromServer", EventLevel.Error, message);

                    return retVal;
                }

                Array.Resize(ref computerId, (int)computerIdLength);

                userInfo.ComputerId = UnicodeEncoding.Unicode.GetString(computerId);

                string userInfoStr = DigitalRightControl.EncryptObjectToStr<UserInfo>(userInfo);

                Stopwatch stopWatch = new Stopwatch();
                stopWatch.Start();

                uint accessFlags = 0;

                retVal = WebFormServices.GetSharedFilePermission(userInfoStr, ref keyStr, ref ivStr, ref accessFlags,ref lastError);

                stopWatch.Stop();

                if (!retVal)
                {
                    string message = "Get file " + messageSend.FileName + " permission from server return error:" + lastError;
                    cacheUserAccessInfo.lastError = message;
                    cacheUserAccessInfo.accessStatus = false;

                    EventManager.WriteMessage(293, "GetAccessPermissionFromServer", EventLevel.Error, message);

                    return retVal;
                }
                else
                {
                    string message = "Get file " + messageSend.FileName + " permission frome server return succeed, spent " + stopWatch.ElapsedMilliseconds + " milliseconds.";
                    EventManager.WriteMessage(208, "GetAccessPermissionFromServer", EventLevel.Verbose, message);
                }

                cacheUserAccessInfo.key = keyStr;
                cacheUserAccessInfo.iv = ivStr;
                cacheUserAccessInfo.accessFlags = accessFlags;

            }
            catch (Exception ex)
            {
                EventManager.WriteMessage(286, "GetAccessPermissionFromServer", EventLevel.Error, "Get file " + messageSend.FileName + "permission failed with exception:" + ex.Message);
                retVal = false;
            }
            finally
            {
                gcHandle.Free();
            }

            return retVal;

        }

        static public bool GetSharedFileAccessPermission(FilterAPI.MessageSendData messageSend, ref FilterAPI.MessageReplyData messageReply)
        {
            Boolean retVal = true;
            string userPassword = string.Empty;
            string fileName = messageSend.FileName;
            string lastError = string.Empty;
            string processName = string.Empty;
            string userName = string.Empty;
            bool isFirstAccess = false;
            CacheUserAccessInfo cacheUserAccessInfo = new CacheUserAccessInfo();

            try
            {

                FilterAPI.DecodeProcessName(messageSend.ProcessId, out processName);
                FilterAPI.DecodeUserName(messageSend.Sid, out userName);

                DRPolicyData drPolicyData = new DRPolicyData();
                retVal = GetDRPolicyDataFromByteArray(messageSend.DataBuffer, messageSend.Length, ref drPolicyData, ref lastError);
                if (!retVal)
                {
                    EventManager.WriteMessage(258, "GetUserPermission", EventLevel.Error, "Process encrypted file failed because of error:" + lastError);
                    return false;
                }

                if (((uint)drPolicyData.AESFlags & (uint)AESFlags.Flags_Request_IV_And_Key_From_User) != (uint)AESFlags.Flags_Request_IV_And_Key_From_User)
                {
                    //this is not the shared encrypted file, return false here.
                    EventManager.WriteMessage(260, "GetUserPermission", EventLevel.Error, fileName + " is not the shared encrypted file, return false here.");
                    return false;
                }

                string index = (userName + "_" + processName + "_" + Utils.ByteArrayToHexStr(drPolicyData.IV)).ToLower();

                //cache the same user/process/filename access.
                lock (userAccessCache)
                {
                    if (userAccessCache.ContainsKey(index))
                    {
                        cacheUserAccessInfo = userAccessCache[index];
                        EventManager.WriteMessage(446, "GetUserPermission", EventLevel.Verbose, "Thread" + Thread.CurrentThread.ManagedThreadId + ",userInfoKey " + index + " exists in the cache table.");
                    }
                    else
                    {
                        isFirstAccess = true;
                        cacheUserAccessInfo.index = index;
                        cacheUserAccessInfo.lastAccessTime = DateTime.Now;
                        userAccessCache.Add(index, cacheUserAccessInfo);
                        EventManager.WriteMessage(435, "GetUserPermission", EventLevel.Verbose, "Thread" + Thread.CurrentThread.ManagedThreadId + ",add userInfoKey " + index + " to the cache table.");
                    }
                }

                //synchronize the same file access.
                if (!cacheUserAccessInfo.syncEvent.WaitOne(new TimeSpan(0, 0, (int)GlobalConfig.ConnectionTimeOut)))
                {
                    string info = "User name: " + userName + ",processname:" + processName + ",file name:" + fileName + " wait for permission timeout.";
                    EventManager.WriteMessage(402, "GetUserPermission", EventLevel.Warning, info);
                }

                TimeSpan timeSpan = DateTime.Now - cacheUserAccessInfo.lastAccessTime;

                if (!isFirstAccess && timeSpan.TotalSeconds < cacheTimeOutInSeconds)
                {
                    //the access was cached, return the last access status.
                    retVal = cacheUserAccessInfo.accessStatus;

                    if (!retVal)
                    {
                        EventManager.WriteMessage(308, "GetAccessPermissionFromServer", EventLevel.Error, cacheUserAccessInfo.lastError);
                    }
                    else
                    {
                        string info = "thread" + Thread.CurrentThread.ManagedThreadId + ",  Cached userInfoKey " + index + " in the cache table,return " + retVal;
                        EventManager.WriteMessage(451, "GetUserPermission", EventLevel.Verbose, info);
                    }

                    return retVal;
                }


                retVal = GetAccessPermissionFromServer(messageSend, drPolicyData, userName, processName, userPassword, ref cacheUserAccessInfo);

                cacheUserAccessInfo.accessStatus = retVal;

            }
            catch (Exception ex)
            {
                EventManager.WriteMessage(340, "GetUserPermission", EventLevel.Error, "filter callback exception." + ex.Message);
                retVal = false;
            }
            finally
            {
                if (retVal && !string.IsNullOrEmpty(cacheUserAccessInfo.key))
                {
                    byte[] encryptKey = Utils.ConvertHexStrToByteArray(cacheUserAccessInfo.key);
                    byte[] encryptIV = Utils.ConvertHexStrToByteArray(cacheUserAccessInfo.iv);


                    //write the iv and key to the reply data buffer with format FilterAPI.AESDataBuffer
                    MemoryStream ms = new MemoryStream(messageReply.DataBuffer);
                    BinaryWriter bw = new BinaryWriter(ms);
                    if (messageSend.MessageType == (uint)FilterAPI.FilterCommand.FILTER_REQUEST_ENCRYPTION_IV_AND_KEY_AND_ACCESSFLAG)
                    {
                        bw.Write(cacheUserAccessInfo.accessFlags);
                    }
                    else
                    {
                        bw.Write((uint)0);
                    }
                    bw.Write(cacheUserAccessInfo.accessFlags);
                    bw.Write(encryptIV.Length);
                    bw.Write(encryptIV);
                    bw.Write(encryptKey.Length);
                    bw.Write(encryptKey);

                    //this is the total AESDataLength
                    messageReply.DataBufferLength = (uint)ms.Position;
                }

                cacheUserAccessInfo.lastAccessTime = DateTime.Now;
                cacheUserAccessInfo.syncEvent.Set();


            }

            return retVal;

        }

        static private void deleteCachedItemTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {

            try
            {
                List<string> keysToRemove = new List<string>();

                foreach (KeyValuePair<string, CacheUserAccessInfo> userItem in userAccessCache)
                {

                    TimeSpan tsSinceLastAccess = DateTime.Now - userItem.Value.lastAccessTime;

                    if (tsSinceLastAccess.TotalSeconds >= cacheTimeOutInSeconds)
                    {
                        keysToRemove.Add(userItem.Key);
                    }
                }

                foreach (string key in keysToRemove)
                {
                    lock (userAccessCache)
                    {
                        userAccessCache.Remove(key);

                        EventManager.WriteMessage(573, "deleteCachedItemTimer_Elapsed", EventLevel.Verbose, "Delete cached item " + key);
                    }
                }
            }
            catch (System.Exception ex)
            {
                EventManager.WriteMessage(46, "deleteCachedItemTimer_Elapsed", EventLevel.Error, "Delete cached item failed with error:" + ex.Message);
            }

        }


    }
}
