﻿///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 2011 EaseFilter Technologies
//    All Rights Reserved
//
//    This software is part of a licensed software product and may
//    only be used or copied in accordance with the terms of that license.
//
//    NOTE:  THIS MODULE IS UNSUPPORTED SAMPLE CODE
//
//    This module contains sample code provided for convenience and
//    demonstration purposes only,this software is provided on an 
//    "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
//     either express or implied.  
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Microsoft.Win32.SafeHandles;
using System.Runtime.InteropServices;
using System.Security.Principal;
using System.Threading;
using System.Diagnostics;
using System.Management;
using System.Collections;
using System.Windows.Forms;

using EaseFilter.CommonObjects;

namespace EaseFilter.FolderLocker
{

    public class FilterWorker
    {
        public enum FilterRuleType
        {
            FolderLocker = 0,
            ShareFolder,
        }    

        public static bool StartService(ref string lastError)
        {
            //Purchase a license key with the link: http://www.easefilter.com/Order.htm
            //Email us to request a trial key: info@easefilter.com //free email is not accepted.
            string registerKey = GlobalConfig.registerKey;

            bool ret = false;
            lastError = string.Empty;

            try
            {

                ret = FilterAPI.StartFilter((int)GlobalConfig.FilterConnectionThreads
                                                , registerKey
                                                , new FilterAPI.FilterDelegate(FilterCallback)
                                                , new FilterAPI.DisconnectDelegate(DisconnectCallback)
                                                , ref lastError);
                if (!ret)
                {
                    lastError = "Start filter service failed with error " + lastError;
                    EventManager.WriteMessage(43, "StartFilter", EventLevel.Error, lastError);
                    return ret;
                }

                SetShareFolderFilterRule();

                EventManager.WriteMessage(102, "StartFilter", EventLevel.Information, "Start filter service succeeded.");
            }
            catch (Exception ex)
            {
                lastError = "Start filter service failed with error " + ex.Message;
                EventManager.WriteMessage(104, "StartFilter", EventLevel.Error, lastError);
            }

            return ret;
        }

        public static void SetShareFolderFilterRule()
        {
            if (!string.IsNullOrEmpty(GlobalConfig.ShareFolder))
            {
                List<string> shareRules = new List<string>();
                foreach (FilterRule rule in GlobalConfig.FilterRules.Values)
                {
                    //remove the old share folder.
                    if (rule.Type == (int)FilterRuleType.ShareFolder)
                    {
                        shareRules.Add(rule.IncludeFileFilterMask);
                    }
                }

                foreach (string shareFolderName in shareRules)
                {
                    GlobalConfig.RemoveFilterRule(shareFolderName);
                }

                //set the new share folder.

                FilterRule filterRule = new FilterRule();
                filterRule.IncludeFileFilterMask = GlobalConfig.ShareFolder + "\\*";
                filterRule.AccessFlag |= ((uint)FilterAPI.AccessFlag.ENABLE_FILE_ENCRYPTION_RULE | (uint)FilterAPI.ALLOW_MAX_RIGHT_ACCESS) & (uint)(~FilterAPI.AccessFlag.ALLOW_ENCRYPT_NEW_FILE);
                filterRule.EncryptMethod = (int)FilterAPI.EncryptionMethod.ENCRYPT_FILE_WITH_SAME_KEY_AND_UNIQUE_IV;
                filterRule.EncryptionPassPhrase = DigitalRightControl.PassPhrase;
                filterRule.Type = (int)FilterRuleType.ShareFolder;
                GlobalConfig.AddFilterRule(filterRule);

                FilterRule filterRule2 = new FilterRule();
                filterRule2.IncludeFileFilterMask = GlobalConfig.ShareFolder + "\\*" + GlobalConfig.ShareFileExt;
                filterRule2.AccessFlag = FilterAPI.ALLOW_MAX_RIGHT_ACCESS;
                filterRule2.Type = (int)FilterRuleType.ShareFolder;
                filterRule2.EventType = (uint)FilterAPI.EVENTTYPE.CREATED | (uint)FilterAPI.EVENTTYPE.RENAMED;
                GlobalConfig.AddFilterRule(filterRule2);

                GlobalConfig.SaveConfigSetting();

            }
        }

        static void DisconnectCallback()
        {
            EventManager.WriteMessage(697, "DisconnectCallback", EventLevel.Information, "Filter Disconnected." + FilterAPI.GetLastErrorMessage());
        }


       static  public bool StopService()
        {
            FilterAPI.StopFilter();
            GlobalConfig.Stop();

            return true;
        }


        static private Boolean FilterCallback(IntPtr sendDataPtr, IntPtr replyDataPtr)
        {
            bool retVal = true;

            try
            {
                FilterAPI.MessageSendData messageSend = new FilterAPI.MessageSendData();
                messageSend = (FilterAPI.MessageSendData)Marshal.PtrToStructure(sendDataPtr, typeof(FilterAPI.MessageSendData));

                if (FilterAPI.MESSAGE_SEND_VERIFICATION_NUMBER != messageSend.VerificationNumber)
                {
                    EventManager.WriteMessage(139, "FilterCallback", EventLevel.Error, "Received message corrupted.Please check if the MessageSendData structure is correct.");
                    return false;
                }

                if (messageSend.MessageType == (uint)FilterAPI.FilterCommand.FILTER_SEND_FILE_CHANGED_EVENT
                     && messageSend.FileName.EndsWith(GlobalConfig.ShareFileExt))
                {
                    string lastError = string.Empty;

                    if (!DigitalRightControl.ProcessSecureShareFile(messageSend.FileName, out lastError))
                    {
                        EventManager.WriteMessage(147, "ProcessSecureShareFile", EventLevel.Error, lastError);
                    }
                    else
                    {
                        EventManager.WriteMessage(159, "ProcessSecureShareFile", EventLevel.Verbose, "Process secure file " + messageSend.FileName + " succeeded.");
                    }
                }
                else if (   messageSend.MessageType == (uint)FilterAPI.FilterCommand.FILTER_REQUEST_ENCRYPTION_IV_AND_KEY
                        || messageSend.MessageType == (uint)FilterAPI.FilterCommand.FILTER_REQUEST_ENCRYPTION_IV_AND_KEY_AND_ACCESSFLAG)
                {

                    if ((replyDataPtr.ToInt64() != 0))
                    {
                        FilterAPI.MessageReplyData messageReply = (FilterAPI.MessageReplyData)Marshal.PtrToStructure(replyDataPtr, typeof(FilterAPI.MessageReplyData));
                        messageReply.MessageId = messageSend.MessageId;
                        messageReply.MessageType = messageSend.MessageType;

                        //since there are a lot of access from the same user/process for the same file,
                        //you can cache the file access here to reduce the server load.

                        //get permission for secure shared file from server, here just demo the server in local,
                        //in reality, your server could be in remote computer.
                        retVal = DRServer.GetSharedFileAccessPermission(messageSend, ref messageReply);

                        if (retVal)
                        {
                            messageReply.ReturnStatus = (uint)FilterAPI.NTSTATUS.STATUS_SUCCESS;
                        }
                        else
                        {
                            //comple the PRE_CREATE,user get access denied for the file open.
                            messageReply.ReturnStatus = (uint)FilterAPI.NTSTATUS.STATUS_ACCESS_DENIED;
                            messageReply.FilterStatus = (uint)FilterAPI.FilterStatus.FILTER_COMPLETE_PRE_OPERATION;
                        }

                        Marshal.StructureToPtr(messageReply, replyDataPtr, true);

                        if (!retVal)
                        {
                            messageSend.Status = (uint)FilterAPI.NTSTATUS.STATUS_ACCESS_DENIED;
                        }                      

                    }
                }

            }
            catch (Exception ex)
            {
                EventManager.WriteMessage(134, "FilterCallback", EventLevel.Error, "filter callback exception." + ex.Message);
                return false;
            }

            return retVal;

        }

    }

}
