﻿///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 2011 EaseFilter Technologies
//    All Rights Reserved
//
//    This software is part of a licensed software product and may
//    only be used or copied in accordance with the terms of that license.
//
//    NOTE:  THIS MODULE IS UNSUPPORTED SAMPLE CODE
//
//    This module contains sample code provided for convenience and
//    demonstration purposes only,this software is provided on an 
//    "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
//     either express or implied.  
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using EaseFilter.CommonObjects;

namespace EaseFilter.FolderLocker
{
    public partial class ShareFileForm : Form
    {
        DRPolicy selectedDRPolicy = null;

        public ShareFileForm(DRPolicy drPolicy)
        {
            InitializeComponent();

            textBox_FileName.Text = drPolicy.FileName.Replace(".psf","");
            textBox_OutputFilePath.Text = drPolicy.FileName.Replace(".psf","");

            selectedDRPolicy = drPolicy;
            textBox_FileName.Enabled = false;
            button_OpenFile.Enabled = false;

            dateTimePicker_ExpireDate.Value = DateTime.FromFileTime(drPolicy.ExpireTime);
            dateTimePicker_ExpireTime.Value = DateTime.FromFileTime(drPolicy.ExpireTime);
            textBox_FileAccessFlags.Text = drPolicy.AccessFlags.ToString();
            textBox_authorizedProcessNames.Text = drPolicy.AuthorizedProcessNames;
            textBox_UnauthorizedProcessNames.Text = drPolicy.UnauthorizedProcessNames;
            textBox_AuthorizedUserNames.Text = drPolicy.AuthorizedUserNames;
            textBox_UnauthorizedUserNames.Text = drPolicy.UnauthorizedUserNames;
            textBox_AuthorizedComputerIds.Text = drPolicy.AuthorizedComputerIds;

            SetCheckBoxValue();
        }

        public ShareFileForm()
        {
            InitializeComponent();
            dateTimePicker_ExpireDate.Value = DateTime.Now.AddDays(1);
            dateTimePicker_ExpireTime.Value = DateTime.Now;
            textBox_FileAccessFlags.Text = FilterAPI.ALLOW_MAX_RIGHT_ACCESS.ToString();
            SetCheckBoxValue();
        }

        private DRPolicyData GetDRPolicyData()
        {
            DRPolicyData policyData = new DRPolicyData();

            policyData.AESVerificationKey = FilterAPI.AES_TAG_KEY;
            policyData.AccessFlags = uint.Parse(textBox_FileAccessFlags.Text);
            
            policyData.AESFlags = AESFlags.Flags_Request_IV_And_Key_From_User;
            if (policyData.AccessFlags > 0)
            {
                policyData.AESFlags |= AESFlags.Flags_Enabled_Check_AccessFlags | AESFlags.Flags_Request_AccessFlags_From_User;
            }

            policyData.LengthOfAccountName = (uint)AccountForm.accountName.Length * 2;
            policyData.AccountName = AccountForm.accountName;

        
            return policyData;

        }

        private DRPolicy GetDRSetting()
        {
            DRPolicy drPolicy = new DRPolicy();

            try
            {
                drPolicy.AuthorizedProcessNames = textBox_authorizedProcessNames.Text.Trim().ToLower();
                drPolicy.UnauthorizedProcessNames = textBox_UnauthorizedProcessNames.Text.Trim().ToLower();
                drPolicy.AuthorizedUserNames = textBox_AuthorizedUserNames.Text.Trim().ToLower();
                drPolicy.UnauthorizedUserNames = textBox_UnauthorizedUserNames.Text.Trim().ToLower();
                drPolicy.AuthorizedComputerIds = textBox_AuthorizedComputerIds.Text.Trim();
                DateTime expireDate = dateTimePicker_ExpireDate.Value.Date + dateTimePicker_ExpireTime.Value.TimeOfDay;
                drPolicy.ExpireTime = expireDate.ToFileTime();
                drPolicy.FileName = Path.GetFileName(textBox_OutputFilePath.Text);
                drPolicy.AccessFlags = uint.Parse(textBox_FileAccessFlags.Text);

                if (null != selectedDRPolicy)
                {
                    drPolicy.EncryptionIV = selectedDRPolicy.EncryptionIV;
                }
            }
            catch (Exception ex)
            {
                throw new Exception("Apply digital right failed with error:" + ex.Message);
            }

            return drPolicy;
        }

        private bool AddNewFileDRInfoToServer(ref string iv, ref string key, ref long creationTime)
        {
            bool retVal = false;
            string lastError = string.Empty;

            try
            {

                iv = string.Empty;
                key = string.Empty;
                creationTime = 0;

                DRPolicy drPolicy = GetDRSetting();

                string encryptedDRPolicy = DigitalRightControl.EncryptObjectToStr<DRPolicy>(drPolicy);

                retVal = WebFormServices.AddShareFile(AccountForm.accountName, AccountForm.password, encryptedDRPolicy, ref creationTime, ref key, ref iv, ref lastError);
                if (!retVal)
                {
                    MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                    MessageBox.Show("Create share encrypted file failed with error:" + lastError, "Process share encrypted file", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return retVal;
                }
            }
            catch (Exception ex)
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show("Get encryption key info failed with error:" + ex.Message, "GetEncryptionKeyAndIVFromServer", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            return retVal;
        }

        private bool CreateOrModifyShareEncryptFile()
        {
            string lastError = string.Empty;            
            string fileName = textBox_FileName.Text.Trim();

            try
            {
                if (fileName.Length == 0)
                {
                    MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                    MessageBox.Show("The file name can't be empty.", "Create share encrypted file", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }

                DateTime expireDateTime = dateTimePicker_ExpireDate.Value.Date + dateTimePicker_ExpireTime.Value.TimeOfDay;
                if (expireDateTime <= DateTime.Now)
                {
                    MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                    MessageBox.Show("The expire time can't be less than current time.", "Create share encrypted file", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }

                if (selectedDRPolicy != null)
                {
                    DRPolicy drPolicy = GetDRSetting();
                    string encryptedDRPolicy = DigitalRightControl.EncryptObjectToStr<DRPolicy>(drPolicy);

                    if (WebFormServices.ModifySharedFileDRInfo(AccountForm.accountName, AccountForm.password, encryptedDRPolicy, ref lastError))
                    {
                        MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                        MessageBox.Show("Modify shared file " + textBox_FileName.Text + " policy succeeded.", "Modify shared file", MessageBoxButtons.OK, MessageBoxIcon.Information);

                        return true;
   
                    }
                    else
                    {                        
                        MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                        MessageBox.Show("Modify shared file " + textBox_FileName.Text + " policy failed with error:" + lastError, "Modify shared file", MessageBoxButtons.OK, MessageBoxIcon.Error);

                        return false;

                    }
                }

                string key = string.Empty;
                string iv = string.Empty;
                long creationTime = DateTime.Now.ToFileTime();

                if (!AddNewFileDRInfoToServer(ref iv, ref key, ref creationTime))
                {
                    return false;
                }

                byte[] encryptIV = null;
                byte[] encryptKey = null;

                if (iv.Length > 0 && key.Length > 0)
                {
                    encryptIV = Utils.ConvertHexStrToByteArray(iv);
                    encryptKey = Utils.ConvertHexStrToByteArray(key);
                }

                DRPolicyData policyData = GetDRPolicyData();
                policyData.IVLength = 16;
                policyData.IV = encryptIV;
                policyData.EncryptionKeyLength = 0;
                policyData.EncryptionKey = new byte[32];
                

                if (!DigitalRightControl.EncryptFileWithEmbeddedDRPolicy(textBox_FileName.Text, textBox_OutputFilePath.Text, encryptIV, encryptKey, policyData,out lastError))
                {
                    MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                    MessageBox.Show("Create share encrypted file " + textBox_FileName.Text + " failed with error:" + lastError, "Process share encrypted file", MessageBoxButtons.OK, MessageBoxIcon.Error);

                    string ivStr = Utils.ByteArrayToHexStr(encryptIV);
                    WebFormServices.DeleteShareFile(AccountForm.accountName, AccountForm.password, ivStr, ref lastError);

                    return false;
                }
                else
                {
                    MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                    MessageBox.Show("Create encrypted file " + textBox_OutputFilePath.Text + " successfully, you can distribute this file to your users.", "Create share file", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    return true;

                }
            }
            catch (Exception ex)
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show("Create share file failed with error " + ex.Message, "Create share encrypted file", MessageBoxButtons.OK, MessageBoxIcon.Error);

                return false;
            }
        }


        private void button_OpenFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();

            if (openFileDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                textBox_FileName.Text = openFileDialog.FileName;
                textBox_OutputFilePath.Text = openFileDialog.FileName + GlobalConfig.ShareFileExt;
            }
        }

      

        private void textBox_FileName_TextChanged(object sender, EventArgs e)
        {
            textBox_OutputFilePath.Text = textBox_FileName.Text + GlobalConfig.ShareFileExt;
        }    
       

        private void SetCheckBoxValue()
        {

            uint accessFlags = uint.Parse(textBox_FileAccessFlags.Text);

            if ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_FILE_ACCESS_FROM_NETWORK) > 0)
            {
                checkBox_AllowRemoteAccess.Checked = true;
            }
            else
            {
                checkBox_AllowRemoteAccess.Checked = false;
            }

            if ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_FILE_DELETE) > 0)
            {
                checkBox_AllowDelete.Checked = true;
            }
            else
            {
                checkBox_AllowDelete.Checked = false;
            }

            if ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_FILE_RENAME) > 0)
            {
                checkBox_AllowRename.Checked = true;
            }
            else
            {
                checkBox_AllowRename.Checked = false;
            }

            if ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_WRITE_ACCESS) > 0
                && (accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_SET_INFORMATION) > 0)
            {
                checkBox_AllowChange.Checked = true;
            }
            else
            {
                checkBox_AllowChange.Checked = false;
            }

            if ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_OPEN_WITH_CREATE_OR_OVERWRITE_ACCESS) > 0)
            {
                checkBox_AllowNewFileCreation.Checked = true;
            }
            else
            {
                checkBox_AllowNewFileCreation.Checked = false;
            }

            if ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_FILE_MEMORY_MAPPED) > 0)
            {
                checkBox_AllowExecution.Checked = true;
            }
            else
            {
                checkBox_AllowExecution.Checked = false;
            }

            if ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_SET_SECURITY_ACCESS) > 0)
            {
                checkBox_AllowSetSecurity.Checked = true;
            }
            else
            {
                checkBox_AllowSetSecurity.Checked = false;
            }


            if ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_ALL_SAVE_AS) > 0)
            {
                checkBox_AllowSaveAs.Checked = true;
            }
            else
            {
                checkBox_AllowSaveAs.Checked = false;
            }

            if ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_READ_ENCRYPTED_FILES) > 0)
            {
                checkBox_AllowReadEncryptedFiles.Checked = true;
            }
            else
            {
                checkBox_AllowReadEncryptedFiles.Checked = false;
            }
        }

        private void checkBox_AllowDelete_CheckedChanged(object sender, EventArgs e)
        {

            uint accessFlags = uint.Parse(textBox_FileAccessFlags.Text);

            if (!checkBox_AllowDelete.Checked)
            {
                accessFlags = accessFlags & ((uint)~FilterAPI.AccessFlag.ALLOW_FILE_DELETE);
            }
            else
            {
                accessFlags = accessFlags | ((uint)FilterAPI.AccessFlag.ALLOW_FILE_DELETE);
            }

            textBox_FileAccessFlags.Text = accessFlags.ToString();
        }

        private void checkBox_AllowChange_CheckedChanged(object sender, EventArgs e)
        {
            uint accessFlags = uint.Parse(textBox_FileAccessFlags.Text);
            if (!checkBox_AllowChange.Checked)
            {
                accessFlags = accessFlags & ((uint)~FilterAPI.AccessFlag.ALLOW_WRITE_ACCESS) & ((uint)~FilterAPI.AccessFlag.ALLOW_SET_INFORMATION);
            }
            else
            {
                accessFlags = accessFlags | ((uint)FilterAPI.AccessFlag.ALLOW_WRITE_ACCESS) | ((uint)FilterAPI.AccessFlag.ALLOW_SET_INFORMATION);
            }

            textBox_FileAccessFlags.Text = accessFlags.ToString();
        }

        private void checkBox_AllowRename_CheckedChanged(object sender, EventArgs e)
        {
            uint accessFlags = uint.Parse(textBox_FileAccessFlags.Text);

            if (!checkBox_AllowRename.Checked)
            {
                accessFlags = accessFlags & ((uint)~FilterAPI.AccessFlag.ALLOW_FILE_RENAME);
            }
            else
            {
                accessFlags = accessFlags | ((uint)FilterAPI.AccessFlag.ALLOW_FILE_RENAME);
            }

            textBox_FileAccessFlags.Text = accessFlags.ToString();
        }

        private void checkBox_AllowRemoteAccess_CheckedChanged(object sender, EventArgs e)
        {
            uint accessFlags = uint.Parse(textBox_FileAccessFlags.Text);

            if (!checkBox_AllowRemoteAccess.Checked)
            {
                accessFlags = accessFlags & ((uint)~FilterAPI.AccessFlag.ALLOW_FILE_ACCESS_FROM_NETWORK);
            }
            else
            {
                accessFlags = accessFlags | ((uint)FilterAPI.AccessFlag.ALLOW_FILE_ACCESS_FROM_NETWORK);
            }

            textBox_FileAccessFlags.Text = accessFlags.ToString();
        }

        private void checkBox_AllowNewFileCreation_CheckedChanged(object sender, EventArgs e)
        {
            uint accessFlags = uint.Parse(textBox_FileAccessFlags.Text);

            if (!checkBox_AllowNewFileCreation.Checked)
            {
                accessFlags = accessFlags & ((uint)~FilterAPI.AccessFlag.ALLOW_OPEN_WITH_CREATE_OR_OVERWRITE_ACCESS);
            }
            else
            {
                accessFlags = accessFlags | ((uint)FilterAPI.AccessFlag.ALLOW_OPEN_WITH_CREATE_OR_OVERWRITE_ACCESS);
            }

            textBox_FileAccessFlags.Text = accessFlags.ToString();
        }

        private void checkBox_AllowExecution_CheckedChanged(object sender, EventArgs e)
        {
            uint accessFlags = uint.Parse(textBox_FileAccessFlags.Text);

            if (!checkBox_AllowExecution.Checked)
            {
                accessFlags = accessFlags & ((uint)~FilterAPI.AccessFlag.ALLOW_FILE_MEMORY_MAPPED);
            }
            else
            {
                accessFlags = accessFlags | ((uint)FilterAPI.AccessFlag.ALLOW_FILE_MEMORY_MAPPED);
            }

            textBox_FileAccessFlags.Text = accessFlags.ToString();
        }

        private void checkBox_AllowSetSecurity_CheckedChanged(object sender, EventArgs e)
        {
            uint accessFlags = uint.Parse(textBox_FileAccessFlags.Text);

            if (!checkBox_AllowSetSecurity.Checked)
            {
                accessFlags = accessFlags & ((uint)~FilterAPI.AccessFlag.ALLOW_FILE_MEMORY_MAPPED);
            }
            else
            {
                accessFlags = accessFlags | ((uint)FilterAPI.AccessFlag.ALLOW_FILE_MEMORY_MAPPED);
            }

            textBox_FileAccessFlags.Text = accessFlags.ToString();
        }


   
        private void checkBox_AllowSaveAs_CheckedChanged(object sender, EventArgs e)
        {
            uint accessFlags = uint.Parse(textBox_FileAccessFlags.Text);

            if (!checkBox_AllowSaveAs.Checked)
            {
                accessFlags = accessFlags & ((uint)~FilterAPI.AccessFlag.ALLOW_ALL_SAVE_AS);
            }
            else
            {
                accessFlags = accessFlags | ((uint)FilterAPI.AccessFlag.ALLOW_ALL_SAVE_AS);
            }

            textBox_FileAccessFlags.Text = accessFlags.ToString();
        }

 
        private void checkBox_AllowReadEncryptedFiles_CheckedChanged(object sender, EventArgs e)
        {
            uint accessFlags = uint.Parse(textBox_FileAccessFlags.Text.Trim());
            if (checkBox_AllowReadEncryptedFiles.Checked)
            {
                accessFlags |= (uint)FilterAPI.AccessFlag.ALLOW_READ_ENCRYPTED_FILES;
            }
            else
            {
                accessFlags &= ~(uint)FilterAPI.AccessFlag.ALLOW_READ_ENCRYPTED_FILES;
            }

            textBox_FileAccessFlags.Text = accessFlags.ToString();
        }

        private void button_FileAccessFlags_Click(object sender, EventArgs e)
        {
            OptionForm optionForm = new OptionForm(OptionForm.OptionType.ShareFileAccessFlag, textBox_FileAccessFlags.Text);

            if (optionForm.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                if (optionForm.AccessFlags > 0)
                {
                    textBox_FileAccessFlags.Text = optionForm.AccessFlags.ToString();
                }
                else
                {
                    //if the accessFlag is 0, it is exclude filter rule,this is not what we want, so we need to include this flag.
                    textBox_FileAccessFlags.Text = ((uint)FilterAPI.AccessFlag.LEAST_ACCESS_FLAG).ToString();
                }

                SetCheckBoxValue();
            }
        }


        private void button_Apply_Click(object sender, EventArgs e)
        {
            try
            {
                string lastError = string.Empty;
                if (CreateOrModifyShareEncryptFile())
                {
                    this.Close();
                }

            }
            catch (Exception ex)
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show("Add digital right information to file " + textBox_OutputFilePath.Text + " failed with error " + ex.Message, "Create", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void button_GetComputerId_Click(object sender, EventArgs e)
        {
            bool retVal = false;
            string myComputerId = string.Empty;
            string lastError = string.Empty;

            retVal = FilterAPI.GetUniqueComputerId(ref myComputerId, ref lastError);

            if (retVal)
            {
                textBox_AuthorizedComputerIds.Text = myComputerId;
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show("This is your local computer unique id:\r\n\r\n" + myComputerId, "Get computer info", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show(lastError, "Get computer info", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            return;
        }

      

    }
}
