﻿///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 2011 EaseFilter Technologies
//    All Rights Reserved
//
//    This software is part of a licensed software product and may
//    only be used or copied in accordance with the terms of that license.
//
//    NOTE:  THIS MODULE IS UNSUPPORTED SAMPLE CODE
//
//    This module contains sample code provided for convenience and
//    demonstration purposes only,this software is provided on an 
//    "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
//     either express or implied.  
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using EaseFilter.CommonObjects;

namespace EaseFilter.FolderLocker
{
    public partial class Form_FolderLocker :Form
    {
        DRPolicy selectedDRPolicy = new DRPolicy();
        public Dictionary<string, DRPolicy> sharedFileList = new Dictionary<string,DRPolicy>();

        public void InitShareFileListView()
        {
            toolStripTextBox_SharedFileFolder.Text = GlobalConfig.ShareFolder;         

            listView_SharedFiles.Clear();
            //create column header for ListView
            listView_SharedFiles.Columns.Add("FileName", 150, System.Windows.Forms.HorizontalAlignment.Left);
            listView_SharedFiles.Columns.Add("CreationTime",100, System.Windows.Forms.HorizontalAlignment.Left);
            listView_SharedFiles.Columns.Add("ExpireTime",100, System.Windows.Forms.HorizontalAlignment.Left);
            listView_SharedFiles.Columns.Add("AuthorizedProcessNames",100, System.Windows.Forms.HorizontalAlignment.Left);
            listView_SharedFiles.Columns.Add("UnauthorizedProcessNames",100, System.Windows.Forms.HorizontalAlignment.Left);
            listView_SharedFiles.Columns.Add("AuthorizedUserNames", 100, System.Windows.Forms.HorizontalAlignment.Left);
            listView_SharedFiles.Columns.Add("UnauthorizedUserNames",100, System.Windows.Forms.HorizontalAlignment.Left);
            listView_SharedFiles.Columns.Add("Copyable", 70, System.Windows.Forms.HorizontalAlignment.Left);
            listView_SharedFiles.Columns.Add("Readable", 70, System.Windows.Forms.HorizontalAlignment.Left);
            listView_SharedFiles.Columns.Add("Writable", 70, System.Windows.Forms.HorizontalAlignment.Left);
            listView_SharedFiles.Columns.Add("Deletable", 70, System.Windows.Forms.HorizontalAlignment.Left);
            listView_SharedFiles.Columns.Add("Renamable", 70, System.Windows.Forms.HorizontalAlignment.Left);

            try
            {
                foreach (DRPolicy sharedFile in sharedFileList.Values)
                {
                    int index = 0;
                    ListViewItem lvItem = new ListViewItem();
                    string[] listEntry = new string[listView_SharedFiles.Columns.Count];

                    string fileName = sharedFile.FileName;

                    if (fileName.EndsWith(GlobalConfig.ShareFileExt))
                    {
                        fileName = fileName.Replace(GlobalConfig.ShareFileExt, "");
                    }

                    listEntry[index++] = fileName;
                    long dateTimeN = sharedFile.CreationTime;
                    DateTime dateTimeD = DateTime.FromFileTime(dateTimeN);
                    listEntry[index++] = String.Format("{0:F}", dateTimeD);
                    dateTimeN = sharedFile.ExpireTime;
                    dateTimeD = DateTime.FromFileTime(dateTimeN);
                    listEntry[index++] = String.Format("{0:F}", dateTimeD);
                    listEntry[index++] = sharedFile.AuthorizedProcessNames;
                    listEntry[index++] = sharedFile.UnauthorizedProcessNames;
                    listEntry[index++] = sharedFile.AuthorizedUserNames;
                    listEntry[index++] = sharedFile.UnauthorizedUserNames;

                    uint accessFlags = sharedFile.AccessFlags;
                    listEntry[index++] = ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_COPY_PROTECTED_FILES_OUT) > 0).ToString();
                    listEntry[index++] = ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_READ_ACCESS) > 0).ToString();
                    listEntry[index++] = ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_WRITE_ACCESS) > 0).ToString();
                    listEntry[index++] = ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_FILE_DELETE) > 0).ToString();
                    listEntry[index++] = ((accessFlags & (uint)FilterAPI.AccessFlag.ALLOW_FILE_RENAME) > 0).ToString();

                    lvItem = new ListViewItem(listEntry, 0);
                    lvItem.Tag = sharedFile;

                    int insertIndex = 0;
                    if (listView_SharedFiles.Items.Count > 0)
                    {
                        for (insertIndex = 0; insertIndex < listView_SharedFiles.Items.Count; insertIndex++)
                        {
                            DRPolicy sharedFileInList = (DRPolicy)(listView_SharedFiles.Items[insertIndex]).Tag;

                            if (sharedFile.CreationTime < sharedFileInList.CreationTime)
                            {
                                break;
                            }
                        }
                     
                    }

                    listView_SharedFiles.Items.Insert(insertIndex,lvItem);

                }
            }
            catch
            {
            }
        }

        public bool GetSharedFileList()
        {
            string lastError = string.Empty;
            string encryptFileList = string.Empty;

            sharedFileList.Clear();

            bool retVal = WebFormServices.GetFileList(AccountForm.accountName, AccountForm.password, ref encryptFileList, ref lastError);
            if (!retVal)
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show(lastError, "GetFileList", MessageBoxButtons.OK, MessageBoxIcon.Error);

                return false;
            }
            else
            {
                if (encryptFileList.Length > 0)
                {
                    List<DRPolicy> decrypFileList = DigitalRightControl.DecryptStrToObject<List<DRPolicy>>(encryptFileList);
                    sharedFileList.Clear();

                    foreach (DRPolicy drPolicy in decrypFileList)
                    {
                        sharedFileList.Add(drPolicy.EncryptionIV, drPolicy);
                    }
                }

                InitShareFileListView();

            }

            return true;
        }

        private void toolStripButton_CreateShareFile_Click(object sender, EventArgs e)
        {
            if (!AccountForm.isAuthorized)
            {
                AccountForm accountForm = new AccountForm();
                accountForm.ShowDialog();
            }

            if (!AccountForm.isAuthorized)
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show("Sign in failed!", "SignIn", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            ShareFileForm shareFileForm = new ShareFileForm();

            if (shareFileForm.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                GetSharedFileList();
                InitShareFileListView();
            }

        }

        private void toolStripButton_SignIn_Click(object sender, EventArgs e)
        {
            if (!AccountForm.isAuthorized)
            {
                AccountForm accountForm = new AccountForm();
                accountForm.ShowDialog();

                if (!AccountForm.isAuthorized)
                {
                    return;
                }
            }
            else
            {
                //MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                //MessageBox.Show("You already signed in.", "SignIn", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }

            GetSharedFileList();
            InitShareFileListView();
        }

        private void toolStripButton_RemoveShareFile_Click(object sender, EventArgs e)
        {
            if (!AccountForm.isAuthorized)
            {
                AccountForm accountForm = new AccountForm();
                accountForm.ShowDialog();
            }

            if (!AccountForm.isAuthorized)
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show("Sign in failed!", "SignIn", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            else
            {
                if (listView_SharedFiles.SelectedItems.Count != 1)
                {
                    MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                    MessageBox.Show("Please select a file.", "Delete", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                DRPolicy drPolicy = (DRPolicy)listView_SharedFiles.SelectedItems[0].Tag;
                string lastError = string.Empty;

                if (!WebFormServices.DeleteShareFile(AccountForm.accountName, AccountForm.password, drPolicy.EncryptionIV, ref lastError))
                {
                    MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                    MessageBox.Show("Delete shared file " + selectedDRPolicy.FileName + " failed with error:" + lastError, "DeleteSharedFile", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }


                GetSharedFileList();
                InitShareFileListView();
            }
        }

        private void toolStripButton_ModifyShareFile_Click(object sender, EventArgs e)
        {
            if (!AccountForm.isAuthorized)
            {
                AccountForm accountForm = new AccountForm();
                accountForm.ShowDialog();
            }

            if (!AccountForm.isAuthorized)
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show("Sign in failed!", "SignIn", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            else
            {
                if (listView_SharedFiles.SelectedItems.Count != 1)
                {
                    MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                    MessageBox.Show("Please select a file.", "Edit", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                DRPolicy drPolicy = (DRPolicy)listView_SharedFiles.SelectedItems[0].Tag;

                ShareFileForm shareFileForm = new ShareFileForm(drPolicy);
                shareFileForm.Text = "EaseFilter Shared File Modification";

                if (shareFileForm.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    GetSharedFileList();
                    InitShareFileListView();
                }
            }
        }

        private void toolStripButton_AccessLog_Click(object sender, EventArgs e)
        {
            if (!AccountForm.isAuthorized)
            {
                AccountForm accountForm = new AccountForm();
                accountForm.ShowDialog();
            }

            if (!AccountForm.isAuthorized)
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show("Sign in failed!", "SignIn", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            SharedFileAccessLogForm accessLog = new SharedFileAccessLogForm();
            accessLog.ShowDialog();


        }



        private void toolStripTextBox_SharedFileFolder_TextChanged(object sender, EventArgs e)
        {

            GlobalConfig.ShareFolder = toolStripTextBox_SharedFileFolder.Text;

            FilterWorker.SetShareFolderFilterRule();
        }


        private void toolStripButton_SetShareFolder_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            if (fbd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                toolStripTextBox_SharedFileFolder.Text = fbd.SelectedPath;

                GlobalConfig.ShareFolder = fbd.SelectedPath;

                FilterWorker.SetShareFolderFilterRule();
            }
        }

        public void RefreshSharedFilesInClient()
        {

            listView_SharedFilesInClient.Clear();
            listView_SharedFilesInClient.Columns.Add("FileName", 250, System.Windows.Forms.HorizontalAlignment.Left);

             if (!string.IsNullOrEmpty(GlobalConfig.ShareFolder))
            {             
                if (!Directory.Exists(GlobalConfig.ShareFolder))
                {
                    Directory.CreateDirectory(GlobalConfig.ShareFolder);
                }
                else
                {
                    string[] sharedFilesNeedToProccess = Directory.GetFiles(GlobalConfig.ShareFolder, "*" + GlobalConfig.ShareFileExt);

                    EventManager.WriteMessage(305, "RefreshSharedFilesInClient", EventLevel.Information, "Total shared file " + sharedFilesNeedToProccess.Length + " end with extension:" + GlobalConfig.ShareFileExt);

                    foreach (string sharedFile in sharedFilesNeedToProccess)
                    {
                        if (sharedFile.EndsWith(GlobalConfig.ShareFileExt))
                        {
                            string lastError = string.Empty;
                            if (DigitalRightControl.ProcessSecureShareFile(sharedFile, out lastError))
                            {
                                EventManager.WriteMessage(305, "RefreshSharedFilesInClient", EventLevel.Information, "Convert shared file " + sharedFile + " to local encrypted file.");
                            }
                            else
                            {
                                EventManager.WriteMessage(312, "RefreshSharedFilesInClient", EventLevel.Information, "Convert shared file " + sharedFile + " to local encrypted file failed with error:" + lastError);
                            }
                        }
                    }

                    string[] sharedFiles = Directory.GetFiles(GlobalConfig.ShareFolder);
                    foreach (string file in sharedFiles)
                    {
                        byte[] iv = null;
                        string lastError = string.Empty;
                        uint aesFlag = 0;

                        if (FilterAPI.GetIVTag(file, ref iv,ref aesFlag, out lastError))
                        {
                            if ((aesFlag & (uint)AESFlags.Flags_Request_IV_And_Key_From_User) != (uint)AESFlags.Flags_Request_IV_And_Key_From_User)
                            {
                                //the encrypted file doesn't require encryption key from server.
                                //this is not our shared files.
                                continue;
                            }

                            ListViewItem lvItem = new ListViewItem();
                            string[] listEntry = new string[listView_SharedFilesInClient.Columns.Count];

                            listEntry[0] = file;

                            lvItem = new ListViewItem(listEntry, 0);
                            lvItem.Tag = listEntry;

                            listView_SharedFilesInClient.Items.Add(lvItem);
                        }
                    }
                }
             }
        }

        private void toolStripButton_RefreshList_Click(object sender, EventArgs e)
        {
            RefreshSharedFilesInClient();
        }

        private void toolStripTextBox_SharedFileFolder_Enter(object sender, EventArgs e)
        {
            GlobalConfig.ShareFolder = toolStripTextBox_SharedFileFolder.Text;
            FilterWorker.SetShareFolderFilterRule();
        }

        private void toolStripTextBox_SharedFileFolder_Leave(object sender, EventArgs e)
        {
            GlobalConfig.ShareFolder = toolStripTextBox_SharedFileFolder.Text;
            FilterWorker.SetShareFolderFilterRule();
        }

    }
}
